@@ -6,20 +6,19 @@ in the source distribution for its full text.
 */
 
 #include "FunctionBar.h"
-
 #include "CRT.h"
+#include "RichString.h"
+#include "XAlloc.h"
 
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
-#include <stdbool.h>
-#include <curses.h>
 
 /*{
-#include "Object.h"
+
+#include <stdbool.h>
 
 typedef struct FunctionBar_ {
-   Object super;
    int size;
    char** functions;
    char** keys;
@@ -29,39 +28,42 @@ typedef struct FunctionBar_ {
 
 }*/
 
-#ifdef DEBUG
-char* FUNCTIONBAR_CLASS = "FunctionBar";
-#else
-#define FUNCTIONBAR_CLASS NULL
-#endif
-
 static const char* FunctionBar_FKeys[] = {"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", NULL};
 
 static const char* FunctionBar_FLabels[] = {"      ", "      ", "      ", "      ", "      ", "      ", "      ", "      ", "      ", "      ", NULL};
 
 static int FunctionBar_FEvents[] = {KEY_F(1), KEY_F(2), KEY_F(3), KEY_F(4), KEY_F(5), KEY_F(6), KEY_F(7), KEY_F(8), KEY_F(9), KEY_F(10)};
 
+static const char* FunctionBar_EnterEscKeys[] = {"Enter", "Esc", NULL};
+static int FunctionBar_EnterEscEvents[] = {13, 27};
+
+FunctionBar* FunctionBar_newEnterEsc(const char* enter, const char* esc) {
+   const char* functions[] = {enter, esc, NULL};
+   return FunctionBar_new(functions, FunctionBar_EnterEscKeys, FunctionBar_EnterEscEvents);
+}
+
 FunctionBar* FunctionBar_new(const char** functions, const char** keys, int* events) {
-   FunctionBar* this = malloc(sizeof(FunctionBar));
-   Object_setClass(this, FUNCTIONBAR_CLASS);
-   ((Object*) this)->delete = FunctionBar_delete;
-   this->functions = (char**) functions;
+   FunctionBar* this = xCalloc(1, sizeof(FunctionBar));
+   this->functions = xCalloc(16, sizeof(char*));
+   if (!functions) {
+      functions = FunctionBar_FLabels;
+   }
+   for (int i = 0; i < 15 && functions[i]; i++) {
+      this->functions[i] = xStrdup(functions[i]);
+   }
    if (keys && events) {
       this->staticData = false; 
-      this->functions = malloc(sizeof(char*) * 15);
-      this->keys = malloc(sizeof(char*) * 15);
-      this->events = malloc(sizeof(int) * 15);
+      this->keys = xCalloc(15, sizeof(char*));
+      this->events = xCalloc(15, sizeof(int));
       int i = 0;
       while (i < 15 && functions[i]) {
-         this->functions[i] = strdup(functions[i]);
-         this->keys[i] = strdup(keys[i]);
+         this->keys[i] = xStrdup(keys[i]);
          this->events[i] = events[i];
          i++;
       }
       this->size = i;
    } else {
       this->staticData = true;
-      this->functions = (char**)( functions ? functions : FunctionBar_FLabels );
       this->keys = (char**) FunctionBar_FKeys;
       this->events = FunctionBar_FEvents;
       this->size = 10;
@@ -69,14 +71,15 @@ FunctionBar* FunctionBar_new(const char** functions, const char** keys, int* eve
    return this;
 }
 
-void FunctionBar_delete(Object* cast) {
-   FunctionBar* this = (FunctionBar*) cast;
+void FunctionBar_delete(FunctionBar* this) {
+   for (int i = 0; i < 15 && this->functions[i]; i++) {
+      free(this->functions[i]);
+   }
+   free(this->functions);
    if (!this->staticData) {
       for (int i = 0; i < this->size; i++) {
-         free(this->functions[i]);
          free(this->keys[i]);
       }
-      free(this->functions);
       free(this->keys);
       free(this->events);
    }
@@ -84,11 +87,10 @@ void FunctionBar_delete(Object* cast) {
 }
 
 void FunctionBar_setLabel(FunctionBar* this, int event, const char* text) {
-   assert(!this->staticData);
    for (int i = 0; i < this->size; i++) {
       if (this->events[i] == event) {
          free(this->functions[i]);
-         this->functions[i] = strdup(text);
+         this->functions[i] = xStrdup(text);
          break;
       }
    }
